package commitments;

import commitments.proofs.CommitmentProof;
import commitments.values.CommitableValue;

/**
 * Represents the functionality of a zero knowledge commitment.
 * Given a [value] that inherits from CommitableValue, an object CommitmentProof
 * is generated.
 * Given a pair [value, proof], a commitment is correctly opened if the [proof] is
 * consistent with the [value].
 * 
 * In general, commitments have security properties such as binding and hiding.
 * This class enforces none of them and therefore enables for mock schemes.
 * @author SergioA
 *
 */
public interface Commitment {

	public CommitmentProof generateCommitment(CommitableValue value);
	
	public boolean openCommitment(CommitableValue value, CommitmentProof proof);
	
}
