package communication.bulletinboard;

/**
 * Represents the accepted subjects in the BulletinBoard.
 * @author SergioA
 *
 */
public enum BBT {
	
	/**
	 * Contains ShareholderData items representing the parties involved.
	 */
	SHAREHOLDER_DATA_TAG(0),
	
	/**
	 * (Currently unused) Stores CommitmentProof values.
	 */
	COMMITMENT_TAG(1),
	
	/**
	 * Contains shares of the public key.
	 */
	PUBLIC_KEY_SHARE_TAG(2),
	
	/**
	 * Contains shares of the private key that are inteded to be SECRET (i.e. not public)
	 */
	SECRET_PARTIAL_KEY_SHARE_TAG(3),
	
	/**
	 * A public share of the secret keys (it needs to work along with at least one 
	 * SECRET_PARTIAL_KEY_SHARE_TAG element to remain secret)
	 */
	PUBLIC_PARTIAL_KEY_SHARE_TAG(4),
	
	/**
	 * Contains an ElGamal public key
	 */
	PUBLIC_KEY_TAG(5),
	
	/**
	 * The first part of an ElGamal ciphertext [c1]
	 */
	CIPHERTEXT_FIRST_TAG(6),
	
	/**
	 * The second part of an ElGamal ciphertext [c2]
	 */
	CIPHERTEXT_SECOND_TAG(7);
	
	public final int i;   
	BBT(int i){
        this.i = i;
    }
}
