package communication.bulletinboard;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentLinkedQueue;

import communication.bulletinboard.messages.Message;

/**
 * Represents a group of Message objects identified by a specific BBT tag. The
 * messages are stored in a LinkedList and are retrieved as a whole by the means
 * of an iterator.
 * @author SergioA
 *
 */
public class BulletinSubject {
	
	/**
	 * List of messages (it must be thread safe)
	 */
	private ConcurrentLinkedQueue<Message> messages;
	
	public BulletinSubject() 
	{
		messages = new ConcurrentLinkedQueue<>();
	}
	
	/**
	 * Adds a new message [m] to the list
	 * @param m
	 */
	public void addMessage(Message m)
	{
		messages.add(m);
	}
	
	/**
	 * Returns the iterator containing the messages of the subject
	 * @return
	 */
	public Iterator<Message> getMessageIterator()
	{		
		return messages.iterator();
	}
	
	/**
	 * Verifies if the list of messages is empty
	 * @return true if there are no messages in the list. false otherwise.
	 */
	public boolean isEmpty()
	{
		return messages.isEmpty();
	}
	
	/**
	 * Output format:
	 * FOR EACH MESSAGE: "<"<message type>":"<string representation of the message>">\n"
	 */
	public String toString()
	{
		StringBuilder bulletinSubjectStringBuilder = new StringBuilder();
		
		Iterator<Message> messageIterator = messages.iterator();
		
		while(messageIterator.hasNext())
		{
			Message message = messageIterator.next();
			bulletinSubjectStringBuilder.append("<");
			bulletinSubjectStringBuilder.append(message.getType());
			bulletinSubjectStringBuilder.append(">:");
			bulletinSubjectStringBuilder.append(message.toString());
			bulletinSubjectStringBuilder.append("\n");
		}
		
		return bulletinSubjectStringBuilder.toString();
	}
}
