package encoding.utils;

/**
 * Provides helper conversion methods between different encodings
 * 
 * @author SergioA
 *
 */
public class Utils {

	/**
	 * Represents [bytes] in hexadecimal format.
	 * 
	 * For readability, strings longer than 4 bytes are trimmed to the format:
	 * <first2bytes>"..."<non-printed bytes>"..."<last2bytes>
	 * 
	 * (Note that <first2bytes> and <last2bytes> are both represented as
	 * 4-character strings.
	 * 
	 * @param bytes
	 * @return (trimmed) hexadecimal representation
	 */
	public static String toHexString(byte[] bytes) {
		char[] hexArray = "0123456789ABCDEF".toCharArray();
		char[] hexChars = new char[bytes.length * 2];
		for (int j = 0; j < bytes.length; j++) {
			int v = bytes[j] & 0xFF;
			hexChars[j * 2] = hexArray[v >>> 4];
			hexChars[j * 2 + 1] = hexArray[v & 0x0F];
		}

		int contentStringLength = hexChars.length;
		if (bytes.length > 4) {
			StringBuilder sb = new StringBuilder();

			appendN(sb, hexChars, 0, 4);
			sb.append("...(");
			sb.append(bytes.length - 4);
			sb.append(" bytes more)");
			sb.append("...");
			appendN(sb, hexChars, contentStringLength - 4, 4);

			return sb.toString();
		}

		return new String(hexChars);
	}

	/**
	 * Appends to [builder] [N] characters of [array], from [array[first]] to
	 * [array[first+N-1]]
	 * 
	 * @param builder
	 * @param array
	 * @param first
	 * @param N
	 */
	private static void appendN(StringBuilder builder, char[] array, int first,
			int N) {
		for (int i = first; i < first + N; i++) {
			builder.append(array[i]);
		}
	}

	/**
	 * [Method directly from the CRISES implementation] For convert a String in
	 * hexadecimal form to byte array.
	 * 
	 * @param in
	 *            String in hexadecimal form
	 * @return A new byte array from the String received in the parameter
	 */
	public static byte[] hexStringToByteArray(String in) {

		int len = in.length() >> 1;
		if (len << 1 != in.length())
			len++;// odd number of hexadecimal numbers; need an extra value to
					// fit in a byte
		byte[] out = new byte[len];
		for (int i = in.length() - 1; i > 0; i = i - 2) {
			out[i >> 1] = (byte) Integer.parseInt(
					"0" + in.substring(i - 1, i + 1), 16);
		}
		return out;
	}

}
