package threshold.elgamal.configuration.parameters;

/**
 * Represents the parameters required for supporting a threshold variant of ElGamal.
 * The key generation produces [n] shares of keys.
 * The encryption of a message is indistinct from classic ElGamal.
 * [t] shares are required to decrypt a ciphertext.
 * @author SergioA
 *
 */
public class ThresholdElGamalPublicParameters extends ElGamalPublicParameters 
{
	/**
	 *  Threshold value
	 */
	private int t;
	
	/**
	 *  Total amount of shares
	 */
	private int n;
	
	public ThresholdElGamalPublicParameters(ElGamalPublicParameters egpp, int t, int n) {
		super(egpp.getP(), egpp.getG(), egpp.getQ());
		this.t = t;
		this.n = n;
	}

	/**
	 * Returns the amount of shares required to produce a decryption.
	 * @return T
	 */
	public int getT() {
		return t;
	}

	/**
	 * Returns the amount of shares created.
	 * @return N
	 */
	public int getN() {
		return n;
	}
	
	

}
