package threshold.elgamal.configuration.shareholderinfo;

import communication.bulletinboard.BBT;
import communication.bulletinboard.BulletinBoard;
import communication.bulletinboard.messages.MockSecretMessage;
import communication.bulletinboard.messages.SimpleMessage;
import exceptions.TagNotSupportedException;

/**
 * Represents a minimum working implementation of the ShareholderInfo interface. 
 * @author SergioA
 */
public class BasicShareholderInfo implements ShareholderInfo{
	
	/**
	 * Unique integer identifier of the shareholder
	 */
	public int id;
	
	/**
	 * Initialize the instance
	 * @param id
	 */
	public BasicShareholderInfo(int id) {
		this.id = id;
	}

	@Override
	public int getID() {
		// TODO Auto-generated method stub
		return id;
	}

	/**
	 * Posts a MockSecretMessage encapsulating a SimpleMessage with the content of
	 * [value], under the subject defined by [tag].
	 * 
	 * THIS IS AN INSECURE APPROACH. THE BROKERS HAVE BEEN IMPLEMENTED TO FOLLOW THE
	 * PROTOCOL, BUT ANY UNAUTHORIZED BROKER CAN ACCESS THE MockSecretMessage MESSAGES
	 * POSTED IN THE BULLETIN BOARD.
	 */
	@Override
	public void sendMessage(BBT tag, byte[] value) throws TagNotSupportedException {
		SimpleMessage message = new SimpleMessage(value);
		MockSecretMessage msm = new MockSecretMessage(this, message);
		BulletinBoard.getInstance().addMessage(tag, msm);
	}
	
	/**
	 * Returns the [id] as a String object
	 */
	public String toString()
	{
		StringBuilder sb = new StringBuilder();
		sb.append(id);
		return sb.toString();
	}
	
	/**
	 * Compares the IDs of two BSI instances to determine if they are the same
	 * @param otherSI
	 * @return true if the values match, false otherwise
	 */
	public boolean equals(BasicShareholderInfo otherSI)
	{
		return id == otherSI.getID();
	}

}
