package threshold.elgamal.entities;

import java.math.BigInteger;

/**
 * Represents the pair corresponding to an encryption with ElGamal.
 * Notation summary:
 * c1,c2 : ciphertext
 * g  	 : generator
 * r	 : random value
 * x	 : private key
 * m	 : encrypted message
 * @author SergioA
 */
public class ElGamalCiphertext 
{
	/**
	 * Randomness factor of the ciphertext, of the form [g^r]
	 */
	private BigInteger c1;
	
	/**
	 * Randomized encryption part of the ciphertext, of the form [m*g^(x*r)]
	 */
	private BigInteger c2;
	
	
	/**
	 * Initializes the ciphertext object. The object is immutable.
	 * @param c1
	 * @param c2
	 */
	public ElGamalCiphertext(BigInteger c1, BigInteger c2) {
		super();
		this.c1 = c1;
		this.c2 = c2;
	}

	/**
	 * Returns the first component of the ciphertext
	 * @return c1
	 */
	public BigInteger getC1() {
		return c1;
	}

	/**
	 * Returns the second component of the ciphertext
	 * @return c2
	 */
	public BigInteger getC2() {
		return c2;
	}
	
	
}
