package threshold.elgamal.entities;

import java.math.BigInteger;

import threshold.elgamal.configuration.parameters.ElGamalPublicParameters;

/**
 * Represents a public key for ElGamal encryption, including the public
 * parameters associated.
 * 
 * @author SergioA Notation summary:
 */
public class ElGamalPublicKey {
	/**
	 * ElGamal public key value
	 */
	private BigInteger h;

	/**
	 * ElGamal public parameters. Its definition is part of the protocol and
	 * need to be used for several operations.
	 */
	private ElGamalPublicParameters egpp;

	public ElGamalPublicKey(byte[] rawPublicKey, ElGamalPublicParameters egpp) {
		h = new BigInteger(rawPublicKey);
		this.egpp = egpp;
	}

	/**
	 * Returns the public key
	 * 
	 * @return h
	 */
	public BigInteger getH() {
		return h;
	}

	/**
	 * Returns the public parameters
	 * 
	 * @return egpp
	 */
	public ElGamalPublicParameters getEgpp() {
		return egpp;
	}

}
