package tuplespaces;

/**
 * Abstract structure to support concurrency requirements. The TupleSpace stores
 *   tuples of String values that can be retrieved by using a mask. If a mask produces 
 *   no match, the querying thread/process blocks.
 * @author SergioA
 */
public interface TupleSpace {
	
	/**
	 * Registers a tuple of String values in the tuple space.
	 * @param tuple
	 */
	public void put(String... tuple);
	
	/**
	 * A pattern is a tuple of (possibly null) String values. A tuple [t] matches 
	 *   [pattern] iff:
	 *     - [t] has the same length as [pattern]
	 *     - For each valid index [i]:
	 *       * [pattern[i]] is null OR
	 *       * [pattern[i]] is equal to [t[i]]
	 *       
	 * The matching tuple [t] is REMOVED from the tuple space.
	 * @param pattern
	 * @return a matching tuple [t] within the TupleSpace
	 */
	public String[] get(String... pattern);

	/**
	 * A pattern is a tuple of (possibly null) String values. A tuple [t] matches 
	 *   [pattern] iff:
	 *     - [t] has the same length as [pattern]
	 *     - For each valid index [i]:
	 *       * [pattern[i]] is null OR
	 *       * [pattern[i]] is equal to [t[i]]
	 *       
	 * The matching tuple [t] is NOT REMOVED from the tuple space.
	 * @param pattern
	 * @return a matching tuple [t] within the TupleSpace
	 */
	public String[] read(String... pattern);
	
}
